/* CLI functions */

#define _CLI_C_

#include "project_config.h"
#include "cli_constants.h"

extern DWORD uptime;

static enum { 	CLI_MAIN_MENU, CLI_SHOW_MENU, CLI_SET_MENU, CLI_UTIL_MENU
	} cli_state = CLI_MAIN_MENU;	//CLI states
	

/* CLI main loop state machine */
void CLI_Task( void )
{
	switch ( cli_state ) {
		case (CLI_MAIN_MENU):
			CLI_main_menu();
			break;
		/**/			
		case (CLI_SHOW_MENU):				/* SHOW... states	*/
			CLI_show_menu();
			break;
		/**/
		case (CLI_SET_MENU):				/* SET... states	*/
			CLI_set_menu();
			break;
		/**/	
		case (CLI_UTIL_MENU):				/* Utilities ... states */
			CLI_util_menu();
			break;
		/**/	
		default:
			break;
	}
}

/* root level CLI */
/* switches state to next level menus */	
void CLI_main_menu( void )
{
	BYTE temp;
	
	if(!CharInQueue())
		return;
		
	temp = recvchar();
	
	if ( temp > 0x1f )
		sendchar( temp );						//echo back
	
	switch( temp ) {
		case( 0x0d ):							//"Enter"				
			send_string( cli_prompt_main );
			send_string( press_qm );
			break;
		case( 0x1b ):							//"ESC"
			cli_state = CLI_MAIN_MENU;
			send_string( cli_prompt_main );
			send_string( press_qm );
			break;
		case( 0x31 ):							//"1 - show menu"
			cli_state = CLI_SHOW_MENU;
			send_string( cli_prompt_show );
			send_string( press_qm );
			break;
		case( 0x32 ):							//"2 - set menu"
			cli_state = CLI_SET_MENU;
			send_string( cli_prompt_set );
			send_string( press_qm );
			break;
		case( 0x33 ):							//"5 - Utilities menu"
			cli_state = CLI_UTIL_MENU;
			send_string( cli_prompt_util );
			send_string( press_qm );
			break;
		case( 0x3f ):							//Question mark
			send_string( cli_root_menu_help );
			send_string( cli_prompt_main );
			send_string( press_qm );
			break;
		default:
			send_string( cli_invalid_key );
			send_string( cli_prompt_main );
			send_string( press_qm );
			break;		
	} //end switch( temp )
}

/* show level CLI */
/* state CLI_SHOW_MENU */
void CLI_show_menu( void )
{
 BYTE temp,i;
	
	if(!CharInQueue())
		return;
		
	temp = recvchar();
	
	if ( temp > 0x1f )
		sendchar( temp );					//echo back
	
	switch( temp ) {
		case( 0x0d ):						//"Enter"				
			send_string( cli_prompt_show );
			send_string( press_qm );
			break;
		case( 0x1b ):						//"ESC"
			cli_state = CLI_MAIN_MENU;		//change state to "Main"
			send_string( cli_prompt_main );
			send_string( press_qm );
			break;
		case( 0x31 ):						//show uptime
			uptime_print();
			send_string( cli_prompt_show );
			send_string( press_qm );			
			break;
		case( 0x32 ):						//2 - Show VDD
			VDD_print();
			send_string( cli_prompt_show );
			send_string( press_qm );			
			break;
		case( 0x33 ):						//3 - Show registers
			send_string("\rQuerying the radio. Please wait...");
			if( Xbee_regdump() == FALSE ) {
				send_string("\rError querying modem");
			}
			send_string( cli_prompt_show );
			send_string( press_qm );	
			break;
		case( 0x3f ):													//Question mark
			send_string( cli_show_menu_help );
			send_string( esc_prev_lvl );
			send_string( cli_prompt_show );
			send_string( press_qm );
			break;
		default:
			send_string( cli_invalid_key );
			send_string( cli_prompt_show);
			send_string( press_qm );
			break;		
	} //end switch( temp )
}

/* SET level CLI */
/* state CLI_SET_MENU */
void CLI_set_menu( void )
{
	BYTE temp;
	
	if(!CharInQueue())
		return;
		
	temp = recvchar();
	
	if ( temp > 0x1f )
		sendchar( temp );						//echo back
	
	switch( temp ) {
		case( 0x0d ):							//"Enter"				
			send_string( cli_prompt_set );
			send_string( press_qm );
			break;
		case( 0x1b ):							//"ESC"
			cli_state = CLI_MAIN_MENU;			//change state to "Main"
			send_string( cli_prompt_main );
			send_string( press_qm );
			break;
		case( 0x31 ):							//"1" - reset MAX3421E
			break;
		case( 0x32 ):							//"2" - turn Vbus on
			break;
		case( 0x33 ):							//"3" - turn Vbus off
			break;
		case( 0x3f ):							//Question mark
			send_string( cli_set_menu_help );
			send_string( esc_prev_lvl );
			send_string( cli_prompt_set );
			send_string( press_qm );
			break;
		default:
			send_string( cli_invalid_key );
			send_string( cli_prompt_set );
			send_string( press_qm );
			break;		
	} //end switch( temp )
}
/* util level CLI */
/* state CLI_SHOW_UTIL */
void CLI_util_menu( void )
{
		BYTE temp;
	
	if(!CharInQueue())
		return;
		
	temp = recvchar();
	
	if ( temp > 0x1f )
		sendchar( temp );					//echo back
	
	// send_string( cli_prompt_util);
	switch( temp ) {
		case( 0x0d ):						//"Enter"				
			break;
		case( 0x1b ):						//"ESC"
			cli_state = CLI_MAIN_MENU;		//change state to "Main"
			send_string( cli_prompt_main );
			send_string( press_qm );
			break;
		case( 0x31 ):						//"1 - monitor"
			Cycle_monitor();
			send_string( cli_prompt_util );
			send_string( press_qm );
			break;
		case( 0x32 ):
			break;
		case( 0x3f ):													//Question mark
			send_string( cli_util_menu_help );
			send_string( esc_prev_lvl );
			send_string( cli_prompt_util );
			send_string( press_qm );
			break;
		default:
			send_string( cli_invalid_key );
			send_string( cli_prompt_util );
			send_string( press_qm );
			break;		
	} //end switch( temp )
}

void CLI_init( void )
{
	send_string( cli_banner );
	send_string( cli_root_menu_help );
	send_string( cli_prompt_main );
	send_string( press_qm );
}
/* prints uptime in human readable format	*/
void uptime_print( void )
{
 BYTE i;
 WORD tmp;
 DWORD uptime_copy;
 	send_string("\r\nSystem uptime is "); 
	uptime_copy = uptime;
	for( i = 0; i < 4; i++) {
		tmp = uptime_copy / uptime_format[ i ].divider;
		if( tmp ) {
			send_decword( tmp );
			send_string( uptime_format[ i ].name );
			uptime_copy = uptime_copy - ( tmp * uptime_format[ i ].divider );
		}
	}
	send_string( crlf );
}
WORD VDD_measure( void )
{
 DWORD delay;
 WORD rcode;
	ADCON0 = 0x3d;
	delay = uptime + 1;
	while( uptime < delay ); 	//wait 1 ms
	ADCON0bits.GO = 1;			//start conversion
	while( ADCON0bits.GO );		//wait for conversion to complete
	rcode = ((( unsigned int )ADRESH )<<8 )|( ADRESL );
	ADCON0bits.ADON = 0;
	return( rcode );	
}
/* Measures and outputs VDD	*/
void VDD_print( void )
{
 WORD tmp;
	send_string("\rVDD is: ");
	tmp =  VDD_measure();
	send_decword((WORD)( 12280000/tmp ));
	send_string(" ADRESH/ADRESL = ");
	send_decword( tmp );
}

BOOL Xbee_regdump( void )
{
 char buf[ RESP_MAXLEN ];
 BYTE i,j,tmpdata;
 	i = 0;
 	while( cmd_query[ i ].name ) {
     	if( Xbee_cmdmode_enter() == FALSE ) {	//enter command mode
     		return( FALSE );
     	}
     	send_string( AT_pref );							//send_prefix
     	send_string( cmd_query[ i ].at_command );		//send command
     	sendchar('\r');									//send CR
     	j = 0;
     	while( j < RESP_MAXLEN ) {						//get response
 			if( CharInQueue() ) {
 				tmpdata = recvchar();
 				if( tmpdata != '\r' ) {
	 				buf[ j ] = tmpdata;
	 				j++;
	 			}
	 			else {
	 				buf[ j ] = 0;			//end of response
	 				j = RESP_MAXLEN;
	 			}
 			}
 		}
 		if( Xbee_cmdmode_exit() == FALSE ) {
 			return( FALSE );
 		}
 		send_string( crlf );
 		send_string( cmd_query[ i ].name );
 		send_ram_string( buf );
 		i++;
 		while( PIE1bits.TXIE );				//wait for end of transmission    
 	}// while( cmd_query[ i ].name
 	return( TRUE );	
}

/* Enters monitor mode. Sensors are probed, results sent, systems goes to sleep */
/* Exit by pressing any key	*/
void Cycle_monitor( void )
{
 DWORD delay;		
 WORD wakeup_counter = 0;
 WORD tmp;
	send_string("\rMonitoring sensors. Press any key to stop.");
	while( CharInQueue() == FALSE ) {
		tmp = VDD_measure();
		SL_RQ = 0;					//wake-up
		delay = uptime + 10000;
		while( uptime < delay );
		sendchar('\r');
		send_decword( wakeup_counter );
		sendchar(' ');
		tmp = VDD_measure();
		send_decword( tmp );
		sendchar(' ');
		send_decword( 12280000/tmp );
		SL_RQ = 1;					//sleep						
		delay = uptime + 10000;
		while( uptime < delay );
		wakeup_counter++;
	}
		
}

//end cli.c
